@@ -1,6 +1,8 @@
+from snowflake.ml.jobs._interop.exception_utils import install_exception_display_hooks
 from snowflake.ml.jobs._utils.types import JOB_STATUS
 from snowflake.ml.jobs.decorators import remote
 from snowflake.ml.jobs.job import MLJob
+from snowflake.ml.jobs.job_definition import MLJobDefinition
 from snowflake.ml.jobs.manager import (
     delete_job,
     get_job,
@@ -10,6 +12,9 @@ from snowflake.ml.jobs.manager import (
     submit_from_stage,
 )
 
+# Initialize exception display hooks for remote job error handling
+install_exception_display_hooks()
+
 __all__ = [
     "remote",
     "submit_file",
@@ -20,4 +25,5 @@ __all__ = [
     "MLJob",
     "JOB_STATUS",
     "submit_from_stage",
+    "MLJobDefinition",
 ]
