@@ -26,6 +26,7 @@ class Service(BaseModel):
     name: str
     compute_pool: str
     ingress_enabled: bool
+    min_instances: int
     max_instances: int
     cpu: Optional[str] = None
     memory: Optional[str] = None
@@ -39,6 +40,8 @@ class Service(BaseModel):
 class Input(BaseModel):
     input_stage_location: str
     input_file_pattern: str
+    column_handling: Optional[str] = None
+    params: Optional[str] = None
 
 
 class Output(BaseModel):
@@ -59,6 +62,7 @@ class Job(BaseModel):
     input: Input
     output: Output
     replicas: Optional[int] = None
+    inference_engine_spec: Optional[InferenceEngineSpec] = None
 
 
 class LogModelArgs(BaseModel):
@@ -74,6 +78,7 @@ class HuggingFaceModel(BaseModel):
     task: Optional[str] = None
     tokenizer: Optional[str] = None
     token: Optional[str] = None
+    token_secret_object: Optional[str] = None
     trust_remote_code: Optional[bool] = False
     revision: Optional[str] = None
     hf_model_kwargs: Optional[str] = "{}"
