@@ -4,6 +4,7 @@ from typing import Any, Collection, Iterable, Optional, Union
 
 import cloudpickle
 import numpy as np
+from packaging import version
 
 import snowflake.snowpark._internal.utils as snowpark_utils
 from snowflake import snowpark
@@ -59,7 +60,11 @@ def register_accumulator_udtf(*, session: Session, statement_params: dict[str, A
             ]
         ),
         input_types=[T.BinaryType()],
-        packages=[f"numpy=={np.__version__}", f"cloudpickle=={cloudpickle.__version__}"],
+        packages=[
+            f"numpy=={version.parse(np.__version__).major}.*",
+            f"cloudpickle=={version.parse(cloudpickle.__version__).major}.*",
+        ],
+        imports=[],  # Prevents unnecessary import resolution.
         name=accumulator,
         is_permanent=False,
         replace=True,
@@ -174,7 +179,11 @@ def register_sharded_dot_sum_computer(*, session: Session, statement_params: dic
             ]
         ),
         input_types=[T.ArrayType(), T.IntegerType(), T.IntegerType()],
-        packages=[f"numpy=={np.__version__}", f"cloudpickle=={cloudpickle.__version__}"],
+        packages=[
+            f"numpy=={version.parse(np.__version__).major}.*",
+            f"cloudpickle=={version.parse(cloudpickle.__version__).major}.*",
+        ],
+        imports=[],  # Prevents unnecessary import resolution.
         name=sharded_dot_and_sum_computer,
         is_permanent=False,
         replace=True,
