@@ -100,6 +100,15 @@ class ModelMonitorManager:
             if source_config.baseline
             else (None, None, None)
         )
+        (
+            timestamp_custom_metric_database_name_id,
+            timestamp_custom_metric_schema_name_id,
+            timestamp_custom_metric_table_name_id,
+        ) = (
+            sql_identifier.parse_fully_qualified_name(source_config.timestamp_custom_metric_table)
+            if source_config.timestamp_custom_metric_table
+            else (None, None, None)
+        )
         model_database_name_id, model_schema_name_id, model_name_id = sql_identifier.parse_fully_qualified_name(
             model_monitor_config.model_version.fully_qualified_model_name
         )
@@ -108,6 +117,8 @@ class ModelMonitorManager:
         prediction_class_columns = self._build_column_list_from_input(source_config.prediction_class_columns)
         actual_score_columns = self._build_column_list_from_input(source_config.actual_score_columns)
         actual_class_columns = self._build_column_list_from_input(source_config.actual_class_columns)
+        segment_columns = self._build_column_list_from_input(source_config.segment_columns)
+        custom_metric_columns = self._build_column_list_from_input(source_config.custom_metric_columns)
 
         id_columns = [sql_identifier.SqlIdentifier(column_name) for column_name in source_config.id_columns]
         ts_column = sql_identifier.SqlIdentifier(source_config.timestamp_column)
@@ -123,6 +134,8 @@ class ModelMonitorManager:
             actual_score_columns=actual_score_columns,
             actual_class_columns=actual_class_columns,
             id_columns=id_columns,
+            segment_columns=segment_columns,
+            custom_metric_columns=custom_metric_columns,
         )
 
         self._model_monitor_client.create_model_monitor(
@@ -144,11 +157,16 @@ class ModelMonitorManager:
             prediction_class_columns=prediction_class_columns,
             actual_score_columns=actual_score_columns,
             actual_class_columns=actual_class_columns,
+            segment_columns=segment_columns,
+            custom_metric_columns=custom_metric_columns,
             refresh_interval=model_monitor_config.refresh_interval,
             aggregation_window=model_monitor_config.aggregation_window,
             baseline_database=baseline_database_name_id,
             baseline_schema=baseline_schema_name_id,
             baseline=baseline_name_id,
+            timestamp_custom_metric_database=timestamp_custom_metric_database_name_id,
+            timestamp_custom_metric_schema=timestamp_custom_metric_schema_name_id,
+            timestamp_custom_metric_table=timestamp_custom_metric_table_name_id,
             statement_params=self.statement_params,
         )
         return model_monitor.ModelMonitor._ref(
