@@ -1,8 +1,10 @@
+import json
+import logging
 from types import ModuleType
 from typing import TYPE_CHECKING, Any, Optional, Union
 
 import pandas as pd
-from absl.logging import logging
+import yaml
 
 from snowflake.ml._internal import platform_capabilities, telemetry
 from snowflake.ml._internal.exceptions import error_codes, exceptions
@@ -11,8 +13,13 @@ from snowflake.ml._internal.utils import sql_identifier
 from snowflake.ml.model import model_signature, task, type_hints
 from snowflake.ml.model._client.model import model_impl, model_version_impl
 from snowflake.ml.model._client.ops import metadata_ops, model_ops, service_ops
+from snowflake.ml.model._client.service import (
+    import_model_spec_schema,
+    model_deployment_spec_schema,
+)
 from snowflake.ml.model._model_composer import model_composer
 from snowflake.ml.model._packager.model_meta import model_meta
+from snowflake.ml.model.models import huggingface
 from snowflake.ml.registry._manager import model_parameter_reconciler
 from snowflake.snowpark import exceptions as snowpark_exceptions, session
 from snowflake.snowpark._internal import utils as snowpark_utils
@@ -59,7 +66,7 @@ class ModelManager:
         signatures: Optional[dict[str, model_signature.ModelSignature]] = None,
         sample_input_data: Optional[type_hints.SupportedDataType] = None,
         user_files: Optional[dict[str, list[str]]] = None,
-        code_paths: Optional[list[str]] = None,
+        code_paths: Optional[list[type_hints.CodePathLike]] = None,
         ext_modules: Optional[list[ModuleType]] = None,
         task: type_hints.Task = task.Task.UNKNOWN,
         experiment_info: Optional["ExperimentInfo"] = None,
@@ -170,7 +177,7 @@ class ModelManager:
         signatures: Optional[dict[str, model_signature.ModelSignature]] = None,
         sample_input_data: Optional[type_hints.SupportedDataType] = None,
         user_files: Optional[dict[str, list[str]]] = None,
-        code_paths: Optional[list[str]] = None,
+        code_paths: Optional[list[type_hints.CodePathLike]] = None,
         ext_modules: Optional[list[ModuleType]] = None,
         task: type_hints.Task = task.Task.UNKNOWN,
         experiment_info: Optional["ExperimentInfo"] = None,
@@ -180,6 +187,31 @@ class ModelManager:
         database_name_id, schema_name_id, model_name_id = sql_identifier.parse_fully_qualified_name(model_name)
         version_name_id = sql_identifier.SqlIdentifier(version_name)
 
+        # Check if model is HuggingFace TransformersPipeline with no repo_snapshot_dir
+        # If so, use remote logging via SYSTEM$IMPORT_MODEL
+        if (
+            isinstance(model, huggingface.TransformersPipeline)
+            and model.compute_pool_for_log is not None
+            and (not hasattr(model, "repo_snapshot_dir") or model.repo_snapshot_dir is None)
+        ):
+            logger.info("HuggingFace model has compute_pool_for_log, using remote logging")
+            return self._remote_log_huggingface_model(
+                model=model,
+                model_name=model_name,
+                version_name=version_name,
+                database_name_id=database_name_id,
+                schema_name_id=schema_name_id,
+                model_name_id=model_name_id,
+                version_name_id=version_name_id,
+                comment=comment,
+                conda_dependencies=conda_dependencies,
+                pip_requirements=pip_requirements,
+                target_platforms=target_platforms,
+                options=options,
+                statement_params=statement_params,
+                progress_status=progress_status,
+            )
+
         # TODO(SNOW-2091317): Remove this when the snowpark enables file PUT operation for snowurls
         use_live_commit = (
             not snowpark_utils.is_in_stored_procedure()  # type: ignore[no-untyped-call]
@@ -221,6 +253,7 @@ class ModelManager:
             )
 
         reconciler = model_parameter_reconciler.ModelParameterReconciler(
+            model=model,
             session=self._model_ops._session,
             database_name=self._database_name,
             schema_name=self._schema_name,
@@ -297,19 +330,11 @@ class ModelManager:
             use_live_commit=use_live_commit,
         )
 
-        mv = model_version_impl.ModelVersion._ref(
-            model_ops=model_ops.ModelOperator(
-                self._model_ops._session,
-                database_name=database_name_id or self._database_name,
-                schema_name=schema_name_id or self._schema_name,
-            ),
-            service_ops=service_ops.ServiceOperator(
-                self._service_ops._session,
-                database_name=database_name_id or self._database_name,
-                schema_name=schema_name_id or self._schema_name,
-            ),
-            model_name=model_name_id,
-            version_name=version_name_id,
+        mv = self._create_model_version_ref(
+            database_name_id=database_name_id,
+            schema_name_id=schema_name_id,
+            model_name_id=model_name_id,
+            version_name_id=version_name_id,
         )
 
         progress_status.update("setting model metadata...")
@@ -332,6 +357,73 @@ class ModelManager:
 
         return mv
 
+    def _remote_log_huggingface_model(
+        self,
+        model: huggingface.TransformersPipeline,
+        model_name: str,
+        version_name: str,
+        database_name_id: Optional[sql_identifier.SqlIdentifier],
+        schema_name_id: Optional[sql_identifier.SqlIdentifier],
+        model_name_id: sql_identifier.SqlIdentifier,
+        version_name_id: sql_identifier.SqlIdentifier,
+        comment: Optional[str],
+        conda_dependencies: Optional[list[str]],
+        pip_requirements: Optional[list[str]],
+        target_platforms: Optional[list[type_hints.SupportedTargetPlatformType]],
+        options: Optional[type_hints.ModelSaveOption],
+        statement_params: Optional[dict[str, Any]],
+        progress_status: type_hints.ProgressStatus,
+    ) -> model_version_impl.ModelVersion:
+        """Log HuggingFace model remotely using SYSTEM$IMPORT_MODEL."""
+        if not isinstance(model, huggingface.TransformersPipeline):
+            raise ValueError(
+                f"Model must be a TransformersPipeline object. The provided model is a {type(model)} object"
+            )
+        progress_status.update("preparing remote model logging...")
+        progress_status.increment()
+
+        # Get compute pool from options or use default
+        compute_pool = model.compute_pool_for_log
+        if compute_pool is None:
+            raise ValueError("compute_pool_for_log is required for remote logging")
+
+        # Construct fully qualified model name
+        db_name = database_name_id.identifier() if database_name_id else self._database_name.identifier()
+        schema_name = schema_name_id.identifier() if schema_name_id else self._schema_name.identifier()
+        fq_model_name = f"{db_name}.{schema_name}.{model_name_id.identifier()}"
+
+        # Build YAML spec for import model
+        yaml_content = self._build_import_model_yaml_spec(
+            model=model,
+            fq_model_name=fq_model_name,
+            version_name=version_name,
+            compute_pool=compute_pool,
+            comment=comment,
+            conda_dependencies=conda_dependencies,
+            pip_requirements=pip_requirements,
+            target_platforms=target_platforms,
+        )
+
+        progress_status.update("Remotely logging the model...")
+        progress_status.increment()
+
+        self._model_ops.run_import_model_query(
+            database_name=db_name,
+            schema_name=schema_name,
+            yaml_content=yaml_content,
+            statement_params=statement_params,
+        )
+        progress_status.update("Remotely logged the model")
+        progress_status.increment()
+
+        # Return ModelVersion object
+        return self._create_model_version_ref(
+            database_name_id=database_name_id,
+            schema_name_id=schema_name_id,
+            model_name_id=model_name_id,
+            version_name_id=version_name_id,
+        )
+
     def get_model(
         self,
         model_name: str,
@@ -407,6 +499,130 @@ class ModelManager:
             statement_params=statement_params,
         )
 
+    def _create_model_version_ref(
+        self,
+        database_name_id: Optional[sql_identifier.SqlIdentifier],
+        schema_name_id: Optional[sql_identifier.SqlIdentifier],
+        model_name_id: sql_identifier.SqlIdentifier,
+        version_name_id: sql_identifier.SqlIdentifier,
+    ) -> model_version_impl.ModelVersion:
+        """Create a ModelVersion reference object.
+
+        Args:
+            database_name_id: Database name identifier, falls back to instance database if None.
+            schema_name_id: Schema name identifier, falls back to instance schema if None.
+            model_name_id: Model name identifier.
+            version_name_id: Version name identifier.
+
+        Returns:
+            ModelVersion reference object.
+        """
+        return model_version_impl.ModelVersion._ref(
+            model_ops=model_ops.ModelOperator(
+                self._model_ops._session,
+                database_name=database_name_id or self._database_name,
+                schema_name=schema_name_id or self._schema_name,
+            ),
+            service_ops=service_ops.ServiceOperator(
+                self._service_ops._session,
+                database_name=database_name_id or self._database_name,
+                schema_name=schema_name_id or self._schema_name,
+            ),
+            model_name=model_name_id,
+            version_name=version_name_id,
+        )
+
+    def _build_import_model_yaml_spec(
+        self,
+        model: huggingface.TransformersPipeline,
+        fq_model_name: str,
+        version_name: str,
+        compute_pool: str,
+        comment: Optional[str],
+        conda_dependencies: Optional[list[str]],
+        pip_requirements: Optional[list[str]],
+        target_platforms: Optional[list[type_hints.SupportedTargetPlatformType]],
+    ) -> str:
+        """Build YAML spec for SYSTEM$IMPORT_MODEL.
+
+        Args:
+            model: HuggingFace TransformersPipeline model.
+            fq_model_name: Fully qualified model name.
+            version_name: Model version name.
+            compute_pool: Compute pool name.
+            comment: Optional comment for the model.
+            conda_dependencies: Optional conda dependencies.
+            pip_requirements: Optional pip requirements.
+            target_platforms: Optional target platforms.
+
+        Returns:
+            YAML string representing the import model spec.
+        """
+        # Convert target_platforms to list of strings
+        target_platforms_list = self._convert_target_platforms_to_list(target_platforms)
+
+        # Build HuggingFaceModel spec
+        hf_model = model_deployment_spec_schema.HuggingFaceModel(
+            hf_model_name=model.model,
+            task=model.task,
+            tokenizer=getattr(model, "tokenizer", None),
+            token_secret_object=model.secret_identifier,
+            trust_remote_code=model.trust_remote_code if model.trust_remote_code is not None else False,
+            revision=model.revision,
+            hf_model_kwargs=json.dumps(model.model_kwargs) if model.model_kwargs else "{}",
+        )
+
+        # Build LogModelArgs
+        log_model_args = model_deployment_spec_schema.LogModelArgs(
+            pip_requirements=pip_requirements,
+            conda_dependencies=conda_dependencies,
+            target_platforms=target_platforms_list,
+            comment=comment,
+        )
+
+        # Build ModelSpec
+        model_spec = import_model_spec_schema.ModelSpec(
+            name=import_model_spec_schema.ModelName(
+                model_name=fq_model_name,
+                version_name=version_name,
+            ),
+            hf_model=hf_model,
+            log_model_args=log_model_args,
+        )
+
+        # Build ImportModelSpec
+        import_spec = import_model_spec_schema.ImportModelSpec(
+            compute_pool=compute_pool,
+            models=[model_spec],
+        )
+
+        # Convert to YAML
+        return yaml.safe_dump(import_spec.model_dump(exclude_none=True))
+
+    def _convert_target_platforms_to_list(
+        self, target_platforms: Optional[list[type_hints.SupportedTargetPlatformType]]
+    ) -> Optional[list[str]]:
+        """Convert target_platforms to list of strings.
+
+        Args:
+            target_platforms: List of target platforms (enums or strings).
+
+        Returns:
+            List of platform strings, or None if input is None.
+        """
+        if not target_platforms:
+            return None
+
+        target_platforms_list = []
+        for tp in target_platforms:
+            if hasattr(tp, "value"):
+                # It's an enum, get the value
+                target_platforms_list.append(tp.value)
+            else:
+                # It's already a string
+                target_platforms_list.append(str(tp))
+        return target_platforms_list
+
     def _parse_fully_qualified_name(
         self, model_name: str
     ) -> tuple[
